@@ -1,5 +1,3 @@
-use geodesy::CoordinateTuple as C;
-
 // examples/00-transformations.rs
 
 // Using Rust Geodesy to transform geodata.
@@ -7,7 +5,11 @@ use geodesy::CoordinateTuple as C;
 // cargo run --example 00-transformations
 
 fn main() {
-    // The context is the entry point to all transformation functionality
+    // The CoordinateTuple type is much used, so
+    // we give it a very brief abbreviation
+    use geodesy::CoordinateTuple as C;
+
+    // The context is the entry point to all transformation functionality:
     let mut ctx = geodesy::Context::new();
     // The concept of a "context data structure" will be well known to
     // PROJ users, where the context plays a somewhat free-flowing role,
@@ -17,7 +19,7 @@ fn main() {
     // methods of the context data structure.
 
     // We need some coordinates to test the code. The convenience methods
-    // `gis` an `geo` produces a 4D coordinate tuple and automatically handles
+    // `gis` and `geo` produces a 4D coordinate tuple and automatically handles
     // conversion of the angular parts to radians, and geographical coordinates
     // in latitude/longitude order, to the GIS convention of longitude/latitude.
     let cph = C::gis(12., 55., 0., 0.); // Copenhagen
@@ -25,7 +27,7 @@ fn main() {
     let sth = C::geo(59., 18., 0., 0.); // Stockholm
     let hel = C::geo(60., 25., 0., 0.); // Helsinki
 
-    // `gis` and `geo` has a sibling `raw` which generates coordinate tuples
+    // `gis` and `geo` have a sibling `raw` which generates coordinate tuples
     // from raw numbers, in case your point coordinates are already given in
     // radians. But since a coordinate tuple is really just an array of 4
     // double precision numbers, you can also generate it directly using plain
@@ -41,22 +43,14 @@ fn main() {
     // Since all operations are carried out in place, the array needs to
     // be mutable, hence `let mut`
 
-    // Let's create a transformation element ("an operator"), turning
+    // Let's create a transformation element ("an operation"), turning
     // geographical coordinates into UTM zone 32 coordinates. Since
     // this may go wrong (e.g. due to syntax errors in the operator
     // definition), use the Rust `match` syntax to handle errors.
-    let utm32 = match ctx.operator("utm: {zone: 32}") {
-        Err(e) => return println!("Awful error: {}", e),
-        Ok(op) => op,
+    let utm32 = match ctx.operation("utm: {zone: 32}") {
+        Some(op) => op,
+        None => return println!("{}", ctx.report()),
     };
-
-    // Here's an example of handling bad syntax:
-    println!("Bad syntax example:");
-    let _aargh = match ctx.operator("aargh: {zone: 23}") {
-        Err(e) => (println!("    Deliberate error: {}", e), 0).1,
-        Ok(op) => op,
-    };
-
     // Now, let's use the utm32-operator to transform some data
     ctx.fwd(utm32, &mut data);
 
@@ -68,7 +62,7 @@ fn main() {
     // The inv() method takes us back to geographic coordinates
     ctx.inv(utm32, &mut data);
 
-    // The output is in radians, so we use this convenience method:
+    // The output is in radians, so we use this convenience function:
     C::degrees_all(&mut data);
 
     println!("Roundtrip to geo:");
@@ -76,8 +70,14 @@ fn main() {
         println!("    {:?}", coord);
     }
 
-    // To get rid of roundtrip-roundoff, let's make a fresh version
-    // of the input data for the next example:
+    // Here's an example of handling bad syntax:
+    println!("Bad syntax example:");
+    if ctx.operation("aargh: {zone: 23}").is_none() {
+        println!("Deliberate error - {}", ctx.report());
+    }
+
+    // To get rid of roundtrip-roundoff noise, let's make a fresh
+    // version of the input data for the next example:
     let mut data = [osl, cph, sth, hel];
 
     // Now a slightly more complex case: Transforming the coordinates,
@@ -91,20 +91,23 @@ fn main() {
     let pipeline = "ed50_wgs84: {
         steps: [
             cart: {ellps: intl},
-            helmert: {dx: -87, dy: -96, dz: -120},
+            helmert: {x: -87, y: -96, z: -120},
             cart: {inv: true, ellps: GRS80}
         ]
     }";
 
-    let ed50_wgs84 = match ctx.operator(pipeline) {
-        Err(e) => return println!("Awful error: {}", e),
-        Ok(op) => op,
+    let ed50_wgs84 = ctx.operation(pipeline);
+    if ed50_wgs84.is_none() {
+        println!("ERROR - {}", ctx.report());
+        return;
     };
+    let ed50_wgs84 = ed50_wgs84.unwrap();
 
     // Since the forward transformation goes *from* ed50 to wgs84, we use
     // the inverse method to take us the other way, back in time to ED50
     ctx.inv(ed50_wgs84, &mut data);
-    // Convert internal lon/lat-in-rad to lat/lon-in-deg.
+    // Convert the internal lon/lat-in-radians format to the more human
+    // readable lat/lon-in-degrees - then print
     C::geo_all(&mut data);
     println!("ed50:");
     for coord in data {
