@@ -23,6 +23,24 @@ impl CoordinateTuple {
         CoordinateTuple([first, second, third, fourth])
     }
 
+    /// A `CoordinateTuple` from latitude/longitude/height/time,
+    /// with the angular input in NMEA format: DDDMM.mmmmm
+    #[must_use]
+    pub fn nmea(latitude: f64, longitude: f64, height: f64, time: f64) -> CoordinateTuple {
+        let longitude = CoordinateTuple::nmea_to_dd(longitude);
+        let latitude = CoordinateTuple::nmea_to_dd(latitude);
+        CoordinateTuple([longitude, latitude, height, time]).to_radians()
+    }
+
+    /// A `CoordinateTuple` from latitude/longitude/height/time, with
+    /// the angular input in extended NMEA format: DDDMMSS.sssss
+    #[must_use]
+    pub fn nmeass(latitude: f64, longitude: f64, height: f64, time: f64) -> CoordinateTuple {
+        let longitude = CoordinateTuple::nmeass_to_dd(longitude);
+        let latitude = CoordinateTuple::nmeass_to_dd(latitude);
+        CoordinateTuple::geo(latitude, longitude, height, time)
+    }
+
     /// A `CoordinateTuple` consisting of 4 `NaN`s
     #[must_use]
     pub fn nan() -> CoordinateTuple {
@@ -162,6 +180,84 @@ impl CoordinateTuple {
             .hypot(self[1] - other[1])
             .hypot(self[2] - other[2])
     }
+
+    /// The 3D distance between two points given as internal angular
+    /// coordinates. Mostly a shortcut for test authoring
+    pub fn default_ellps_3d_dist(&self, other: &CoordinateTuple) -> f64 {
+        let e = crate::Ellipsoid::default();
+        e.cartesian(self).hypot3(&e.cartesian(other))
+    }
+
+    /// The Geodesic distance on the default ellipsoid. Mostly a shortcut
+    /// for test authoring
+    pub fn default_ellps_dist(&self, other: &CoordinateTuple) -> f64 {
+        crate::Ellipsoid::default().distance(self, other)
+    }
+
+    /// Simplistic transformation from degrees, minutes and seconds-with-decimals
+    /// to degrees-with-decimals. No sanity check: Sign taken from degree-component,
+    /// minutes forced to unsigned by i16 type, but passing a negative value for
+    /// seconds leads to undefined behaviour.
+    pub fn dms_to_dd(d: i32, m: u16, s: f64) -> f64 {
+        d.signum() as f64 * (d.abs() as f64 + (m as f64 + s / 60.) / 60.)
+    }
+
+    /// Simplistic transformation from degrees and minutes-with-decimals
+    /// to degrees-with-decimals. No sanity check: Sign taken from
+    /// degree-component, but passing a negative value for minutes leads
+    /// to undefined behaviour.
+    pub fn dm_to_dd(d: i32, m: f64) -> f64 {
+        d.signum() as f64 * (d.abs() as f64 + (m as f64 / 60.))
+    }
+
+    /// Simplistic transformation from the NMEA DDDMM.mmm format to
+    /// to degrees-with-decimals. No sanity check: Invalid input,
+    /// such as 5575.75 (where the number of minutes exceed 60) leads
+    /// to undefined behaviour.
+    pub fn nmea_to_dd(nmea: f64) -> f64 {
+        let sign = nmea.signum();
+        let dm = nmea.abs() as u32;
+        let fraction = nmea.abs() - dm as f64;
+        let d = dm / 100;
+        let m = (dm - d * 100) as f64 + fraction;
+        sign * (d as f64 + (m as f64 / 60.))
+    }
+
+    /// Transformation from degrees-with-decimals to the NMEA DDDMM.mmm format.
+    pub fn dd_to_nmea(dd: f64) -> f64 {
+        let sign = dd.signum();
+        let dd = dd.abs();
+        let d = dd.floor();
+        let m = (dd - d) * 60.;
+        sign * (d * 100. + m)
+    }
+
+    /// Simplistic transformation from the extended NMEA DDDMMSS.sss
+    /// format to degrees-with-decimals. No sanity check: Invalid input,
+    /// such as 557575.75 (where the number of minutes and seconds both
+    /// exceed 60) leads to undefined behaviour.
+    pub fn nmeass_to_dd(nmeass: f64) -> f64 {
+        let sign = nmeass.signum();
+        let dms = nmeass.abs() as u32;
+        let fraction = nmeass.abs() - dms as f64;
+        let d = dms / 10000;
+        let ms = dms - d * 10000;
+        let m = ms / 100;
+        let s = (ms - m * 100) as f64 + fraction;
+        sign * (d as f64 + ((s as f64 / 60.) + m as f64) / 60.)
+    }
+
+    /// Transformation from degrees-with-decimals to the extended
+    /// NMEA DDDMMSS.sss format.
+    pub fn dd_to_nmeass(dd: f64) -> f64 {
+        let sign = dd.signum();
+        let dd = dd.abs();
+        let d = dd.floor();
+        let mm = (dd - d) * 60.;
+        let m = mm.floor();
+        let s = (mm - m) * 60.;
+        sign * (d * 10000. + m * 100. + s)
+    }
 }
 
 impl Index<usize> for CoordinateTuple {
@@ -177,84 +273,9 @@ impl IndexMut<usize> for CoordinateTuple {
     }
 }
 
-/*
-#[derive(Clone, Copy, Debug)]
-pub struct CoordType {}
-
-#[derive(Clone, Copy, Debug, Default)]
-pub struct DMS {
-    pub s: f32,
-    pub d: i16,
-    pub m: i8,
-}
-
-#[allow(dead_code)]
-impl DMS {
-    #[must_use]
-    pub fn new(d: i16, m: i8, s: f32) -> DMS {
-        DMS { s, d, m }
-    }
-    #[must_use]
-    pub fn to_degrees(self) -> f64 {
-        (f64::from(self.s) / 60. + f64::from(self.m)) / 60. + f64::from(self.d)
-    }
-    #[must_use]
-    pub fn to_radians(self) -> f64 {
-        self.to_degrees().to_radians()
-    }
-}
-
-#[allow(dead_code)]
-enum CoordinateKind {
-    Linear,
-    Angular,
-    Parametric,
-    Pass,
-}
-
-#[allow(dead_code)]
-enum Coordinate {
-    Northish {
-        from: usize,
-        to: usize,
-        scale: f64,
-        offset: f64,
-        nan: f64,
-        kind: CoordinateKind,
-    },
-    Eastish {},
-    Upish {},
-    Timeish {},
-    Pass {},
-
-    // An `enum` may either be `unit-like`,
-    PageLoad,
-    PageUnload,
-    // like tuple structs,
-    KeyPress(char),
-    Paste(String),
-    // or c-like structures.
-    Click {
-        x: i64,
-        y: i64,
-    },
-}
-*/
-
 #[cfg(test)]
 mod tests {
-    use super::*;
-    /*
-    #[test]
-    fn dms() {
-        let dms = DMS::new(60, 24, 36.);
-        assert_eq!(dms.d, 60);
-        assert_eq!(dms.m, 24);
-        assert_eq!(dms.s, 36.);
-        let d = dms.to_degrees();
-        assert_eq!(d, 60.41);
-    }
-    */
+    use crate::CoordinateTuple;
 
     #[test]
     fn coordinatetuple() {
@@ -264,6 +285,36 @@ mod tests {
         assert_eq!(d[0], 12f64.to_radians());
         let e = d.to_degrees();
         assert_eq!(e[0], c.to_degrees()[0]);
+
+        assert_eq!(CoordinateTuple::dms_to_dd(55, 30, 36.), 55.51);
+        assert_eq!(CoordinateTuple::dm_to_dd(55, 30.60), 55.51);
+
+        // nmea + nmeass
+        assert!((CoordinateTuple::nmea_to_dd(5530.60) - 55.51).abs() < 1e-10);
+        assert!((CoordinateTuple::nmea_to_dd(15530.60) - 155.51).abs() < 1e-10);
+        assert!((CoordinateTuple::nmea_to_dd(-15530.60) + 155.51).abs() < 1e-10);
+        assert!((CoordinateTuple::nmeass_to_dd(553036.0) - 55.51).abs() < 1e-10);
+        assert_eq!(CoordinateTuple::dd_to_nmea(55.5025), 5530.15);
+        assert_eq!(CoordinateTuple::dd_to_nmea(-55.5025), -5530.15);
+        assert_eq!(CoordinateTuple::dd_to_nmeass(55.5025), 553009.);
+        assert_eq!(CoordinateTuple::dd_to_nmeass(-55.51), -553036.);
+
+        assert_eq!(CoordinateTuple::nmea_to_dd(5500.), 55.);
+        assert_eq!(CoordinateTuple::nmea_to_dd(-5500.), -55.);
+        assert_eq!(
+            CoordinateTuple::nmea_to_dd(5530.60),
+            -CoordinateTuple::nmea_to_dd(-5530.60)
+        );
+        assert_eq!(
+            CoordinateTuple::nmeass_to_dd(553036.),
+            -CoordinateTuple::nmeass_to_dd(-553036.00)
+        );
+
+        let lat = CoordinateTuple::dms_to_dd(55, 30, 36.);
+        let lon = CoordinateTuple::dms_to_dd(12, 45, 36.);
+        let dms = CoordinateTuple::geo(lat, lon, 0., 2020.);
+        let geo = CoordinateTuple::geo(55.51, 12.76, 0., 2020.);
+        assert!(geo.default_ellps_dist(&dms) < 1e-10);
     }
 
     #[test]
